/*----------------------------------------------------------------------
John Robbins
Microsoft Systems Journal, April 1998 - Bugslayer Column!
----------------------------------------------------------------------*/
#include "stdafx.h"
#include "CrashFinderDoc.h"
#include "FindCrashDlg.h"
#include "SymbolEngine.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE [] = __FILE__ ;
#endif

BEGIN_MESSAGE_MAP ( CFindCrashDlg , CDialog )
    ON_COMMAND ( IDC_FIND , OnFindAddr )
END_MESSAGE_MAP ( )


CFindCrashDlg :: CFindCrashDlg ( CCrashFinderDoc * pDoc )
               : CDialog ( CFindCrashDlg::IDD ) ,
                 m_cAddress ( )     ,
                 m_cModule ( )      ,
                 m_cFunction ( )    ,
                 m_cFnDispl ( )     ,
                 m_cSourceFile ( )  ,
                 m_cSourceLine ( )  ,
                 m_cSrcDispl ( )
{
    ASSERT ( this ) ;
    ASSERT ( NULL != pDoc ) ;
    m_pDoc = pDoc ;
}

void CFindCrashDlg :: DoDataExchange ( CDataExchange * pDX )
{
    ASSERT ( this ) ;

    CDialog::DoDataExchange ( pDX ) ;
}

BOOL CFindCrashDlg :: OnInitDialog ( void )
{
    ASSERT ( this ) ;

    // Subclass the edit control with my cheezy hex only edit control
    //  class.
    BOOL bRet = m_cHexCtrl.SubclassDlgItem ( IDC_HEXEDIT , this ) ;
    ASSERT ( TRUE == bRet ) ;

    // Attach to all the static text controls on the dialog.
    CWnd * pWnd = GetDlgItem ( IDC_ADDRESS ) ;
    ASSERT ( NULL != pWnd ) ;
    VERIFY ( m_cAddress.Attach ( pWnd->m_hWnd ) ) ;

    pWnd = GetDlgItem ( IDC_MODULE ) ;
    ASSERT ( NULL != pWnd ) ;
    VERIFY ( m_cModule.Attach ( pWnd->m_hWnd ) ) ;

    pWnd = GetDlgItem ( IDC_FUNCTION ) ;
    ASSERT ( NULL != pWnd ) ;
    VERIFY ( m_cFunction.Attach ( pWnd->m_hWnd ) ) ;

    pWnd = GetDlgItem ( IDC_FNDISPLACEMENT ) ;
    ASSERT ( NULL != pWnd ) ;
    VERIFY ( m_cFnDispl.Attach ( pWnd->m_hWnd ) ) ;

    pWnd = GetDlgItem ( IDC_SOURCEFILE ) ;
    ASSERT ( NULL != pWnd ) ;
    VERIFY ( m_cSourceFile.Attach ( pWnd->m_hWnd ) ) ;

    pWnd = GetDlgItem ( IDC_SOURCELINE ) ;
    ASSERT ( NULL != pWnd ) ;
    VERIFY ( m_cSourceLine.Attach ( pWnd->m_hWnd ) ) ;

    pWnd = GetDlgItem ( IDC_SRCDISPLACEMENT ) ;
    ASSERT ( NULL != pWnd ) ;
    VERIFY ( m_cSrcDispl.Attach ( pWnd->m_hWnd ) ) ;

    // Let the normal initialization take place.
    bRet = CDialog::OnInitDialog ( ) ;
    return ( bRet ) ;
}

void CFindCrashDlg :: OnCancel ( )
{
    // Detach all the value controls.
    VERIFY ( m_cAddress.Detach ( )      ) ;
    VERIFY ( m_cModule.Detach ( )       ) ;
    VERIFY ( m_cFunction.Detach ( )     ) ;
    VERIFY ( m_cFnDispl.Detach ( )      ) ;
    VERIFY ( m_cSourceFile.Detach ( )   ) ;
    VERIFY ( m_cSourceLine.Detach ( )   ) ;
    VERIFY ( m_cSrcDispl.Detach ( )     ) ;

    CDialog::OnCancel ( ) ;
}

void CFindCrashDlg :: OnFindAddr ( void )
{
    ASSERT ( this ) ;

    // Get the text out of the edit control.
    TCHAR szTemp[ 25 ] ;

    CEdit * pEdit = (CEdit*)GetDlgItem ( IDC_HEXEDIT ) ;
    ASSERT ( NULL != pEdit ) ;

    pEdit->GetWindowText ( szTemp , sizeof ( szTemp ) ) ;

    // Try and convert it to an value.  I'm taking the easy way out.
    DWORD dwAddr ;
    int iRet = _stscanf ( szTemp        ,
                          _T ( "%x" ) ,
                          &dwAddr       ) ;

    if ( 1 != iRet )
    {
        CString sMsg ;
        sMsg.FormatMessage ( IDS_LOADADDRBADCONVERSION , szTemp ) ;
        AfxMessageBox ( sMsg ) ;
        return ;
    }

    // Now look up the address through the document.
    ASSERT ( NULL != m_pDoc ) ;

    // Since the largest symbol that the MS code generators can handle
    //  is 256 that is all that is needed.
    IMAGEHLP_SYMBOL * pstIHSym =
          (IMAGEHLP_SYMBOL *)new BYTE [ sizeof ( IMAGEHLP_SYMBOL ) +
                                        256                         ] ;
    memset ( pstIHSym , NULL , sizeof ( IMAGEHLP_SYMBOL ) + 256 ) ;
    pstIHSym->SizeOfStruct = sizeof ( IMAGEHLP_SYMBOL ) + 256 ;
    pstIHSym->MaxNameLength = 256 ;

    // The line information.
    CImageHlp_Line stIHLine ;

    // The displacements.
    DWORD dwFnDispl  ;
    DWORD dwSrcDispl ;

    // The module string.
    CString sModule ;

    // Where I build up the test to output.
    CString sTemp   ;

    // Do the look up.
    int iRes = m_pDoc->LookUpAddress ( dwAddr     ,
                                       sModule    ,
                                       *pstIHSym  ,
                                       dwFnDispl  ,
                                       stIHLine   ,
                                       dwSrcDispl  ) ;


    // Always show the address that was just looked up.
    sTemp.Format ( _T ( "0x%08X" ) , dwAddr ) ;
    m_cAddress.SetWindowText ( sTemp ) ;

    // Did a proper module get returned?
    if ( eLUMOD == ( eLUMOD & iRes ) )
    {
        // Pull off the path information so the dialog does not have to
        //  be a mile wide.
        TCHAR szFile[ MAX_PATH ] ;
        TCHAR szExt[ MAX_PATH ] ;

        _tsplitpath ( (LPCTSTR)sModule ,
                      NULL             ,
                      NULL             ,
                      szFile           ,
                      szExt             ) ;

        sModule = szFile ;
        sModule += szExt ;
        m_cModule.SetWindowText ( sModule ) ;
    }
    else
    {
        m_cModule.SetWindowText ( _T ( "" ) ) ;
    }

    // Did a proper symbol get returned?
    if ( eLUSYM == ( eLUSYM & iRes ) )
    {
        m_cFunction.SetWindowText ( (LPCSTR)pstIHSym->Name ) ;

        if ( 0 != dwFnDispl )
        {
            sTemp.Format ( IDS_DISPLACEMENTFMT  ,
                           dwFnDispl            ,
                           dwAddr - dwFnDispl    ) ;
            m_cFnDispl.SetWindowText ( sTemp ) ;
        }
        else
        {
            m_cFnDispl.SetWindowText ( _T ( "" ) ) ;
        }
    }
    else
    {
        VERIFY ( sTemp.LoadString ( IDS_FUNCNOTFOUND ) ) ;
        m_cFunction.SetWindowText ( sTemp ) ;
        m_cFnDispl.SetWindowText ( _T ( "" ) ) ;
    }

    // Did a proper source/line get returned?
    if ( eLULINE == ( eLULINE & iRes ) )
    {
        m_cSourceFile.SetWindowText ( stIHLine.FileName ) ;

        sTemp.Format ( _T ( "%d" ) , stIHLine.LineNumber ) ;
        m_cSourceLine.SetWindowText ( sTemp ) ;

        if ( 0 != dwSrcDispl )
        {
            sTemp.Format ( IDS_DISPLACEMENTFMT  ,
                           dwSrcDispl           ,
                           dwAddr - dwSrcDispl   ) ;
            m_cSrcDispl.SetWindowText ( sTemp ) ;
        }
        else
        {
            m_cSrcDispl.SetWindowText ( _T ( "" ) ) ;
        }
    }
    else
    {
        VERIFY ( sTemp.LoadString ( IDS_LINENOTFOUND ) ) ;
        m_cSourceFile.SetWindowText ( sTemp ) ;
        m_cSourceLine.SetWindowText ( _T ( "" ) ) ;
        m_cSrcDispl.SetWindowText ( _T ( "" ) ) ;
    }

    // I am done so get rid of the memory that I allocated earlier.
    delete [] pstIHSym ;
}
